@@ -1,6 +1,8 @@
-import sys, os
+import sys, os, os.path
 from distutils.core import Extension
 
+from versioninfo import get_base_dir, split_version
+
 try:
     from Cython.Distutils import build_ext as build_pyx
     import Cython.Compiler.Version
@@ -8,13 +10,26 @@ try:
 except ImportError:
     CYTHON_INSTALLED = False
 
-EXT_MODULES = ["lxml.etree", "lxml.objectify","lxml.pyclasslookup"]
+EXT_MODULES = ["lxml.etree", "lxml.objectify"]
 
 PACKAGE_PATH = "src/lxml/"
 
+if sys.version_info[0] >= 3:
+    _system_encoding = sys.getdefaultencoding()
+    if _system_encoding is None:
+        _system_encoding = "iso-8859-1" # :-)
+    def decode_input(data):
+        return data.decode(_system_encoding)
+else:
+    def decode_input(data):
+        return data
+
 def env_var(name):
-    value = os.getenv(name, '')
-    return value.split(os.pathsep)
+    value = os.getenv(name)
+    if value:
+        return decode_input(value).split(os.pathsep)
+    else:
+        return []
 
 def ext_modules(static_include_dirs, static_library_dirs, static_cflags): 
     if CYTHON_INSTALLED:
@@ -63,10 +78,11 @@ def ext_modules(static_include_dirs, static_library_dirs, static_cflags):
     result = []
     for module in modules:
         main_module_source = PACKAGE_PATH + module + source_extension
+        dependencies = find_dependencies(module)
         result.append(
             Extension(
             module,
-            sources = [main_module_source],
+            sources = [main_module_source] + dependencies,
             extra_compile_args = ['-w'] + _cflags,
             define_macros = _define_macros,
             include_dirs = _include_dirs,
@@ -76,6 +92,32 @@ def ext_modules(static_include_dirs, static_library_dirs, static_cflags):
             ))
     return result
 
+def find_dependencies(module):
+    if not CYTHON_INSTALLED:
+        return []
+    from Cython.Compiler.Version import version
+    if split_version(version) < (0,9,6,13):
+        return []
+
+    package_dir = os.path.join(get_base_dir(), PACKAGE_PATH)
+    files = os.listdir(package_dir)
+    pxd_files = [ os.path.join(PACKAGE_PATH, filename) for filename in files
+                  if filename.endswith('.pxd') ]
+
+    if 'etree' in module:
+        pxi_files = [ os.path.join(PACKAGE_PATH, filename)
+                      for filename in files
+                      if filename.endswith('.pxi')
+                      and 'objectpath' not in filename ]
+        pxd_files = [ filename for filename in pxd_files
+                      if 'etreepublic' not in filename ]
+    elif 'objectify' in module:
+        pxi_files = [ os.path.join(PACKAGE_PATH, 'objectpath.pxi') ]
+    else:
+        pxi_files = []
+
+    return pxd_files + pxi_files
+
 def extra_setup_args():
     result = {}
     if CYTHON_INSTALLED:
@@ -177,8 +219,7 @@ def run_command(cmd, *args):
         _ERROR_PRINTED = True
         print("ERROR: %s" % errors)
         print("** make sure the development packages of libxml2 and libxslt are installed **\n")
-    output = rf.read()
-    return (output or '').strip()
+    return decode_input(rf.read()).strip()
 
 def get_library_versions():
     xml2_version = run_command(find_xml2_config(), "--version")
@@ -232,7 +273,12 @@ def has_option(name):
         sys.argv.remove('--%s' % name)
         return True
     except ValueError:
-        return False
+        pass
+    # allow passing all cmd line options also as environment variables
+    env_val = os.getenv(name.upper().replace('-', '_'), 'false').lower()
+    if env_val == "true":
+        return True
+    return False
 
 # pick up any commandline options
 OPTION_WITHOUT_OBJECTIFY = has_option('without-objectify')
