@@ -5,12 +5,16 @@ IO test cases that apply to both etree and ElementTree
 """
 
 import unittest
-import tempfile, gzip, os, gc, shutil
+import tempfile, gzip, os, os.path, sys, gc, shutil
 
-from common_imports import etree, ElementTree, fileInTestDir
-from common_imports import SillyFileLike, LargeFileLike
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
 
-class IOTestCaseBase(unittest.TestCase):
+from common_imports import etree, ElementTree, fileInTestDir, _str, _bytes
+from common_imports import SillyFileLike, LargeFileLike, HelperTestCase
+
+class IOTestCaseBase(HelperTestCase):
     """(c)ElementTree compatibility for IO functions/methods
     """
     etree = None
@@ -45,21 +49,47 @@ class IOTestCaseBase(unittest.TestCase):
         ElementTree = self.etree.ElementTree
     
         element = Element('top')
+        element.text = _str("qwrtioüöä\uAABB")
         tree = ElementTree(element)
         self.buildNodes(element, 10, 3)
-        f = open(self.getTestFilePath('testdump.xml'), 'w')
+        f = open(self.getTestFilePath('testdump.xml'), 'wb')
         tree.write(f, encoding='UTF-8')
         f.close()
-        f = open(self.getTestFilePath('testdump.xml'), 'r')
+        f = open(self.getTestFilePath('testdump.xml'), 'rb')
         tree = ElementTree(file=f)
         f.close()
-        f = open(self.getTestFilePath('testdump2.xml'), 'w')
+        f = open(self.getTestFilePath('testdump2.xml'), 'wb')
         tree.write(f, encoding='UTF-8')
         f.close()
-        f = open(self.getTestFilePath('testdump.xml'), 'r')
+        f = open(self.getTestFilePath('testdump.xml'), 'rb')
         data1 = f.read()
         f.close()
-        f = open(self.getTestFilePath('testdump2.xml'), 'r')
+        f = open(self.getTestFilePath('testdump2.xml'), 'rb')
+        data2 = f.read()
+        f.close()
+        self.assertEquals(data1, data2)
+
+    def test_tree_io_latin1(self):
+        Element = self.etree.Element
+        ElementTree = self.etree.ElementTree
+
+        element = Element('top')
+        element.text = _str("qwrtioüöäßÃ¡")
+        tree = ElementTree(element)
+        self.buildNodes(element, 10, 3)
+        f = open(self.getTestFilePath('testdump.xml'), 'wb')
+        tree.write(f, encoding='iso-8859-1')
+        f.close()
+        f = open(self.getTestFilePath('testdump.xml'), 'rb')
+        tree = ElementTree(file=f)
+        f.close()
+        f = open(self.getTestFilePath('testdump2.xml'), 'wb')
+        tree.write(f, encoding='iso-8859-1')
+        f.close()
+        f = open(self.getTestFilePath('testdump.xml'), 'rb')
+        data1 = f.read()
+        f.close()
+        f = open(self.getTestFilePath('testdump2.xml'), 'rb')
         data2 = f.read()
         f.close()
         self.assertEquals(data1, data2)
@@ -70,7 +100,7 @@ class IOTestCaseBase(unittest.TestCase):
         handle, filename = tempfile.mkstemp(suffix=".xml")
         self.tree.write(filename)
         try:
-            self.assertEqual(open(filename).read().replace('\n', ''),
+            self.assertEqual(open(filename, 'rb').read().replace(_bytes('\n'), _bytes('')),
                              self.root_str)
         finally:
             os.close(handle)
@@ -83,7 +113,7 @@ class IOTestCaseBase(unittest.TestCase):
         f.write(self.root_str)
         f.close()
         try:
-            f_gz = gzip.open(filename, 'r')
+            f_gz = gzip.open(filename, 'rb')
             tree = self.etree.parse(f_gz)
             f_gz.close()
             self.assertEqual(self.etree.tostring(tree.getroot()), self.root_str)
@@ -137,7 +167,7 @@ class IOTestCaseBase(unittest.TestCase):
         handle, filename = tempfile.mkstemp(suffix=".xml")
         os.write(handle, self.root_str)
         try:
-            f = open(filename, 'r')
+            f = open(filename, 'rb')
             tree = self.etree.ElementTree()
             root = tree.parse(f)
             f.close()
@@ -176,7 +206,11 @@ class IOTestCaseBase(unittest.TestCase):
             pass
         class TestFile:
             data = '<root>test</'
-            next_char = iter(data).next
+            try:
+                next_char = iter(data).next
+            except AttributeError:
+                # Python 3
+                next_char = iter(data).__next__
             counter = 0
             def read(self, amount=None):
                 if amount is None:
@@ -185,7 +219,7 @@ class IOTestCaseBase(unittest.TestCase):
                 else:
                     try:
                         self.counter += 1
-                        return self.next_char()
+                        return _bytes(self.next_char())
                     except StopIteration:
                         raise LocalError
         f = TestFile()
@@ -215,4 +249,4 @@ def test_suite():
     return suite
 
 if __name__ == '__main__':
-    print 'to test use test.py %s' % __file__
+    print('to test use test.py %s' % __file__)
