@@ -4,13 +4,29 @@
 Test cases related to XSLT processing
 """
 
-import unittest, copy
+import unittest, copy, sys, os.path
 
-from common_imports import etree, StringIO, HelperTestCase, fileInTestDir
-from common_imports import doctest
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
+
+is_python3 = sys.version_info[0] >= 3
+
+try:
+    unicode = __builtins__["unicode"]
+except (NameError, KeyError): # Python 3
+    unicode = str
+
+try:
+    basestring = __builtins__["basestring"]
+except (NameError, KeyError): # Python 3
+    basestring = str
+
+from common_imports import etree, BytesIO, HelperTestCase, fileInTestDir
+from common_imports import doctest, _bytes, _str, make_doctest
 
 class ETreeXSLTTestCase(HelperTestCase):
-    """XPath tests etree"""
+    """XSLT tests etree"""
         
     def test_xslt(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
@@ -24,12 +40,12 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree)
+        res = st(tree)
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>B</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
 
     def test_xslt_elementtree_error(self):
         self.assertRaises(ValueError, etree.XSLT, etree.ElementTree())
@@ -86,7 +102,8 @@ class ETreeXSLTTestCase(HelperTestCase):
                           str(res))
 
     def test_xslt_utf8(self):
-        tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
+        tree = self.parse(_bytes('<a><b>\\uF8D2</b><c>\\uF8D2</c></a>'
+                                 ).decode("unicode_escape"))
         style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -97,16 +114,21 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree)
-        expected = u'''\
+        res = st(tree)
+        expected = _bytes('''\
 <?xml version="1.0" encoding="UTF-8"?>
-<foo>\uF8D2</foo>
-'''
-        self.assertEquals(expected,
-                          unicode(str(res), 'UTF-8'))
+<foo>\\uF8D2</foo>
+''').decode("unicode_escape")
+        if is_python3:
+            self.assertEquals(expected,
+                              str(bytes(res), 'UTF-8'))
+        else:
+            self.assertEquals(expected,
+                              unicode(str(res), 'UTF-8'))
 
     def test_xslt_encoding(self):
-        tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
+        tree = self.parse(_bytes('<a><b>\\uF8D2</b><c>\\uF8D2</c></a>'
+                                 ).decode("unicode_escape"))
         style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -117,16 +139,21 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree)
-        expected = u'''\
+        res = st(tree)
+        expected = _bytes('''\
 <?xml version="1.0" encoding="UTF-16"?>
-<foo>\uF8D2</foo>
-'''
-        self.assertEquals(expected,
-                          unicode(str(res), 'UTF-16'))
+<foo>\\uF8D2</foo>
+''').decode("unicode_escape")
+        if is_python3:
+            self.assertEquals(expected,
+                              str(bytes(res), 'UTF-16'))
+        else:
+            self.assertEquals(expected,
+                              unicode(str(res), 'UTF-16'))
 
     def test_xslt_encoding_override(self):
-        tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
+        tree = self.parse(_bytes('<a><b>\\uF8D2</b><c>\\uF8D2</c></a>'
+                                 ).decode("unicode_escape"))
         style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -137,19 +164,22 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree)
-        expected = u"""\
+        res = st(tree)
+        expected = _bytes("""\
 <?xml version='1.0' encoding='UTF-16'?>\
-<foo>\uF8D2</foo>"""
+<foo>\\uF8D2</foo>""").decode("unicode_escape")
 
-        f = StringIO()
+        f = BytesIO()
         res.write(f, encoding='UTF-16')
-        result = unicode(f.getvalue(), 'UTF-16').replace('\n', '')
-        self.assertEquals(expected,
-                          result)
+        if is_python3:
+            result = str(f.getvalue(), 'UTF-16').replace('\n', '')
+        else:
+            result = unicode(str(f.getvalue()), 'UTF-16').replace('\n', '')
+        self.assertEquals(expected, result)
 
     def test_xslt_unicode(self):
-        tree = self.parse(u'<a><b>\uF8D2</b><c>\uF8D2</c></a>')
+        tree = self.parse(_bytes('<a><b>\\uF8D2</b><c>\\uF8D2</c></a>'
+                                 ).decode("unicode_escape"))
         style = self.parse('''\
 <xsl:stylesheet version="1.0"
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -160,11 +190,11 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree)
-        expected = u'''\
+        res = st(tree)
+        expected = _bytes('''\
 <?xml version="1.0"?>
-<foo>\uF8D2</foo>
-'''
+<foo>\\uF8D2</foo>
+''').decode("unicode_escape")
         self.assertEquals(expected,
                           unicode(res))
 
@@ -176,7 +206,7 @@ class ETreeXSLTTestCase(HelperTestCase):
     xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
     exclude-result-prefixes="str xsl">
   <xsl:template match="text()">
-    <xsl:value-of select="str:align(string(.), '---', 'center')" />
+    <xsl:value-of select="str:align(string(.), '***', 'center')" />
   </xsl:template>
   <xsl:template match="*">
     <xsl:copy>
@@ -189,10 +219,33 @@ class ETreeXSLTTestCase(HelperTestCase):
         res = st(tree)
         self.assertEquals('''\
 <?xml version="1.0"?>
-<a><b>-B-</b><c>-C-</c></a>
+<a><b>*B*</b><c>*C*</c></a>
 ''',
                           str(res))
 
+    if etree.LIBXSLT_VERSION >= (1,1,21):
+        def test_exslt_str_attribute_replace(self):
+            tree = self.parse('<a><b>B</b><c>C</c></a>')
+            style = self.parse('''\
+      <xsl:stylesheet version = "1.0"
+          xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
+          xmlns:str="http://exslt.org/strings"
+          extension-element-prefixes="str">
+
+          <xsl:template match="/">
+            <h1 class="{str:replace('abc', 'b', 'x')}">test</h1>
+          </xsl:template>
+
+      </xsl:stylesheet>''')
+
+            st = etree.XSLT(style)
+            res = st(tree)
+            self.assertEquals('''\
+<?xml version="1.0"?>
+<h1 class="axc">test</h1>
+''',
+                              str(res))
+
     def test_exslt_math(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
         style = self.parse('''\
@@ -269,12 +322,29 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree, bar="'Bar'")
+        res = st(tree, bar="'Bar'")
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>Bar</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
+
+    def test_xslt_parameter_invalid(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:param name="bar"/>
+  <xsl:template match="/">
+    <foo><xsl:value-of select="$bar" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = self.assertRaises(etree.XSLTApplyError,
+                                st, tree, bar="<test/>")
+        res = self.assertRaises(etree.XSLTApplyError,
+                                st, tree, bar="....")
 
     if etree.LIBXSLT_VERSION < (1,1,18):
         # later versions produce no error
@@ -306,12 +376,12 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree, bar="'Bar'", baz="'Baz'")
+        res = st(tree, bar="'Bar'", baz="'Baz'")
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>Bar</foo><foo>Baz</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
         
     def test_xslt_parameter_xpath(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
@@ -325,12 +395,12 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree, bar="/a/b/text()")
+        res = st(tree, bar="/a/b/text()")
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>B</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
 
         
     def test_xslt_default_parameters(self):
@@ -346,18 +416,18 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         st = etree.XSLT(style)
-        res = st.apply(tree, bar="'Bar'")
+        res = st(tree, bar="'Bar'")
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>Bar</foo>
 ''',
-                          st.tostring(res))
-        res = st.apply(tree)
+                          str(res))
+        res = st(tree)
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>Default</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
         
     def test_xslt_html_output(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
@@ -373,7 +443,7 @@ class ETreeXSLTTestCase(HelperTestCase):
 
         st = etree.XSLT(style)
         res = st(tree)
-        self.assertEquals('''<html><body>B</body></html>''',
+        self.assertEquals('<html><body>B</body></html>',
                           str(res).strip())
 
     def test_xslt_include(self):
@@ -381,7 +451,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         st = etree.XSLT(tree)
 
     def test_xslt_include_from_filelike(self):
-        f = open(fileInTestDir('test1.xslt'), 'r')
+        f = open(fileInTestDir('test1.xslt'), 'rb')
         tree = etree.parse(f)
         f.close()
         st = etree.XSLT(tree)
@@ -398,14 +468,14 @@ class ETreeXSLTTestCase(HelperTestCase):
         source = self.parse(xml)
         styledoc = self.parse(xslt)
         style = etree.XSLT(styledoc)
-        result = style.apply(source)
+        result = style(source)
 
         etree.tostring(result.getroot())
         
         source = self.parse(xml)
         styledoc = self.parse(xslt)
         style = etree.XSLT(styledoc)
-        result = style.apply(source)
+        result = style(source)
         
         etree.tostring(result.getroot())
 
@@ -421,10 +491,10 @@ class ETreeXSLTTestCase(HelperTestCase):
         source = self.parse(xml)
         styledoc = self.parse(xslt)
         transform = etree.XSLT(styledoc)
-        result = transform.apply(source)
-        result = transform.apply(source)
+        result = transform(source)
+        result = transform(source)
         etree.tostring(result.getroot())
-        result = transform.apply(source)
+        result = transform(source)
         etree.tostring(result.getroot())
         str(result)
 
@@ -446,8 +516,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         source = self.parse(xml)
         styledoc = self.parse(xslt)
         style = etree.XSLT(styledoc)
-        result = style.apply(source)
-        self.assertEqual('', style.tostring(result))
+        result = style(source)
         self.assertEqual('', str(result))
 
     def test_xslt_message(self):
@@ -463,8 +532,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         source = self.parse(xml)
         styledoc = self.parse(xslt)
         style = etree.XSLT(styledoc)
-        result = style.apply(source)
-        self.assertEqual('', style.tostring(result))
+        result = style(source)
         self.assertEqual('', str(result))
         self.assert_("TEST TEST TEST" in [entry.message
                                           for entry in style.error_log])
@@ -482,9 +550,8 @@ class ETreeXSLTTestCase(HelperTestCase):
         source = self.parse(xml)
         styledoc = self.parse(xslt)
         style = etree.XSLT(styledoc)
-        result = style.apply(source)
-        self.assertEqual('', style.tostring(result))
-        self.assertEqual('', str(result))
+
+        self.assertRaises(etree.XSLTApplyError, style, source)
         self.assert_("TEST TEST TEST" in [entry.message
                                           for entry in style.error_log])
 
@@ -504,7 +571,7 @@ class ETreeXSLTTestCase(HelperTestCase):
 
         result = tree.xslt(style, bar="'Bar'", baz="'Baz'")
         self.assertEquals(
-            '<doc><foo>Bar</foo><foo>Baz</foo></doc>',
+            _bytes('<doc><foo>Bar</foo><foo>Baz</foo></doc>'),
             etree.tostring(result.getroot()))
         
     def test_multiple_elementrees(self):
@@ -518,30 +585,30 @@ class ETreeXSLTTestCase(HelperTestCase):
 </xsl:stylesheet>''')
 
         self.assertEquals(self._rootstring(tree),
-                          '<a><b>B</b><c>C</c></a>')
+                          _bytes('<a><b>B</b><c>C</c></a>'))
         result = tree.xslt(style)
         self.assertEquals(self._rootstring(tree),
-                          '<a><b>B</b><c>C</c></a>')
+                          _bytes('<a><b>B</b><c>C</c></a>'))
         self.assertEquals(self._rootstring(result),
-                          '<A><B>B</B><C>C</C></A>')
+                          _bytes('<A><B>B</B><C>C</C></A>'))
 
         b_tree = etree.ElementTree(tree.getroot()[0])
         self.assertEquals(self._rootstring(b_tree),
-                          '<b>B</b>')
+                          _bytes('<b>B</b>'))
         result = b_tree.xslt(style)
         self.assertEquals(self._rootstring(tree),
-                          '<a><b>B</b><c>C</c></a>')
+                          _bytes('<a><b>B</b><c>C</c></a>'))
         self.assertEquals(self._rootstring(result),
-                          '<B>B</B>')
+                          _bytes('<B>B</B>'))
 
         c_tree = etree.ElementTree(tree.getroot()[1])
         self.assertEquals(self._rootstring(c_tree),
-                          '<c>C</c>')
+                          _bytes('<c>C</c>'))
         result = c_tree.xslt(style)
         self.assertEquals(self._rootstring(tree),
-                          '<a><b>B</b><c>C</c></a>')
+                          _bytes('<a><b>B</b><c>C</c></a>'))
         self.assertEquals(self._rootstring(result),
-                          '<C>C</C>')
+                          _bytes('<C>C</C>'))
 
     def test_extensions1(self):
         tree = self.parse('<a><b>B</b></a>')
@@ -558,7 +625,7 @@ class ETreeXSLTTestCase(HelperTestCase):
 
         result = tree.xslt(style, {('testns', 'mytext') : mytext})
         self.assertEquals(self._rootstring(result),
-                          '<A>X</A>')
+                          _bytes('<A>X</A>'))
 
     def test_extensions2(self):
         tree = self.parse('<a><b>B</b></a>')
@@ -578,7 +645,108 @@ class ETreeXSLTTestCase(HelperTestCase):
 
         result = tree.xslt(style)
         self.assertEquals(self._rootstring(result),
-                          '<A>X</A>')
+                          _bytes('<A>X</A>'))
+
+    def test_extension_element(self):
+        tree = self.parse('<a><b>B</b></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+    xmlns:myns="testns"
+    extension-element-prefixes="myns"
+    exclude-result-prefixes="myns">
+  <xsl:template match="a">
+    <A><myns:myext>b</myns:myext></A>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        class MyExt(etree.XSLTExtension):
+            def execute(self, context, self_node, input_node, output_parent):
+                child = etree.Element(self_node.text)
+                child.text = 'X'
+                output_parent.append(child)
+
+        extensions = { ('testns', 'myext') : MyExt() }
+
+        result = tree.xslt(style, extensions=extensions)
+        self.assertEquals(self._rootstring(result),
+                          _bytes('<A><b>X</b></A>'))
+
+    def test_extension_element_content(self):
+        tree = self.parse('<a><b>B</b></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+    xmlns:myns="testns"
+    extension-element-prefixes="myns">
+  <xsl:template match="a">
+    <A><myns:myext><x>X</x><y>Y</y><z/></myns:myext></A>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        class MyExt(etree.XSLTExtension):
+            def execute(self, context, self_node, input_node, output_parent):
+                output_parent.extend(list(self_node)[1:])
+
+        extensions = { ('testns', 'myext') : MyExt() }
+
+        result = tree.xslt(style, extensions=extensions)
+        self.assertEquals(self._rootstring(result),
+                          _bytes('<A><y>Y</y><z/></A>'))
+
+    def test_extension_element_apply_templates(self):
+        tree = self.parse('<a><b>B</b></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+    xmlns:myns="testns"
+    extension-element-prefixes="myns">
+  <xsl:template match="a">
+    <A><myns:myext><x>X</x><y>Y</y><z/></myns:myext></A>
+  </xsl:template>
+  <xsl:template match="x" />
+  <xsl:template match="z">XYZ</xsl:template>
+</xsl:stylesheet>''')
+
+        class MyExt(etree.XSLTExtension):
+            def execute(self, context, self_node, input_node, output_parent):
+                for child in self_node:
+                    for result in self.apply_templates(context, child):
+                        if isinstance(result, basestring):
+                            el = etree.Element("T")
+                            el.text = result
+                        else:
+                            el = result
+                        output_parent.append(el)
+
+        extensions = { ('testns', 'myext') : MyExt() }
+
+        result = tree.xslt(style, extensions=extensions)
+        self.assertEquals(self._rootstring(result),
+                          _bytes('<A><T>Y</T><T>XYZ</T></A>'))
+
+    def test_extension_element_raise(self):
+        tree = self.parse('<a><b>B</b></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
+    xmlns:myns="testns"
+    extension-element-prefixes="myns"
+    exclude-result-prefixes="myns">
+  <xsl:template match="a">
+    <A><myns:myext>b</myns:myext></A>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        class MyError(Exception):
+            pass
+
+        class MyExt(etree.XSLTExtension):
+            def execute(self, context, self_node, input_node, output_parent):
+                raise MyError("expected!")
+
+        extensions = { ('testns', 'myext') : MyExt() }
+        self.assertRaises(MyError, tree.xslt, style, extensions=extensions)
 
     def test_xslt_document_XML(self):
         # make sure document('') works from parsed strings
@@ -645,7 +813,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         parser = etree.XMLParser()
         parser.resolvers.add(TestResolver())
 
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(etree.XML(_bytes("""\
 <xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:l="local">
@@ -666,7 +834,7 @@ class ETreeXSLTTestCase(HelperTestCase):
     <l:entry>B</l:entry>
   </l:data>
 </xsl:stylesheet>
-""", parser))
+"""), parser))
 
         self.assertEquals(called['count'], 0)
         result = xslt(etree.XML('<a/>'))
@@ -694,13 +862,36 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(root[3].get("value"),
                           'B')
 
+    def test_xslt_document_parse_allow(self):
+        access_control = etree.XSLTAccessControl(read_file=True)
+        xslt = etree.XSLT(etree.parse(fileInTestDir("test-document.xslt")),
+                          access_control = access_control)
+        result = xslt(etree.XML('<a/>'))
+        root = result.getroot()
+        self.assertEquals(root.tag,
+                          'test')
+        self.assertEquals(root[0].tag,
+                          '{http://www.w3.org/1999/XSL/Transform}stylesheet')
+
+    def test_xslt_document_parse_deny(self):
+        access_control = etree.XSLTAccessControl(read_file=False)
+        xslt = etree.XSLT(etree.parse(fileInTestDir("test-document.xslt")),
+                          access_control = access_control)
+        self.assertRaises(etree.XSLTApplyError, xslt, etree.XML('<a/>'))
+
+    def test_xslt_document_parse_deny_all(self):
+        access_control = etree.XSLTAccessControl.DENY_ALL
+        xslt = etree.XSLT(etree.parse(fileInTestDir("test-document.xslt")),
+                          access_control = access_control)
+        self.assertRaises(etree.XSLTApplyError, xslt, etree.XML('<a/>'))
+
     def test_xslt_move_result(self):
-        root = etree.XML('''\
+        root = etree.XML(_bytes('''\
         <transform>
           <widget displayType="fieldset"/>
-        </transform>''')
+        </transform>'''))
 
-        xslt = etree.XSLT(etree.XML('''\
+        xslt = etree.XSLT(etree.XML(_bytes('''\
         <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
           <xsl:output method="html" indent="no"/>
           <xsl:template match="/">
@@ -713,7 +904,7 @@ class ETreeXSLTTestCase(HelperTestCase):
             <xsl:element name="{@displayType}"/>
           </xsl:template>
 
-        </xsl:stylesheet>'''))
+        </xsl:stylesheet>''')))
 
         result = xslt(root[0])
         root[:] = result.getroot()[:]
@@ -754,12 +945,12 @@ class ETreeXSLTTestCase(HelperTestCase):
                           style_root.tag)
 
         st = etree.XSLT(style_root)
-        res = st.apply(tree)
+        res = st(tree)
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>B</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
 
     def test_xslt_pi_embedded_id(self):
         # test XPath lookup mechanism
@@ -788,12 +979,12 @@ class ETreeXSLTTestCase(HelperTestCase):
                           style_root.tag)
 
         st = etree.XSLT(style_root)
-        res = st.apply(tree)
+        res = st(tree)
         self.assertEquals('''\
 <?xml version="1.0"?>
 <foo>B</foo>
 ''',
-                          st.tostring(res))
+                          str(res))
 
     def test_xslt_pi_get(self):
         tree = self.parse('''\
@@ -878,7 +1069,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals("TEST", pi.get("href"))
 
     def test_exslt_regexp_test(self):
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(etree.XML(_bytes("""\
 <xsl:stylesheet version="1.0"
    xmlns:regexp="http://exslt.org/regular-expressions"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -886,8 +1077,8 @@ class ETreeXSLTTestCase(HelperTestCase):
     <test><xsl:copy-of select="*[regexp:test(string(.), '8.')]"/></test>
   </xsl:template>
 </xsl:stylesheet>
-"""))
-        result = xslt(etree.XML('<a><b>123</b><b>098</b><b>987</b></a>'))
+""")))
+        result = xslt(etree.XML(_bytes('<a><b>123</b><b>098</b><b>987</b></a>')))
         root = result.getroot()
         self.assertEquals(root.tag,
                           'test')
@@ -911,7 +1102,7 @@ class ETreeXSLTTestCase(HelperTestCase):
   </xsl:template>
 </xsl:stylesheet>
 """))
-        result = xslt(etree.XML('<a>abdCdEeDed</a>'))
+        result = xslt(etree.XML(_bytes('<a>abdCdEeDed</a>')))
         root = result.getroot()
         self.assertEquals(root.tag,
                           'test')
@@ -932,7 +1123,7 @@ class ETreeXSLTTestCase(HelperTestCase):
   </xsl:template>
 </xsl:stylesheet>
 """))
-        result = xslt(etree.XML('<a>abdCdEeDed</a>'))
+        result = xslt(etree.XML(_bytes('<a>abdCdEeDed</a>')))
         root = result.getroot()
         self.assertEquals(root.tag,  'test')
         self.assertEquals(len(root), 3)
@@ -956,7 +1147,7 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(root[2][2].text, 'De')
 
     def test_exslt_regexp_match_groups(self):
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(etree.XML(_bytes("""\
 <xsl:stylesheet version="1.0"
    xmlns:regexp="http://exslt.org/regular-expressions"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -969,8 +1160,8 @@ class ETreeXSLTTestCase(HelperTestCase):
     </test>
   </xsl:template>
 </xsl:stylesheet>
-"""))
-        result = xslt(etree.XML('<a/>'))
+""")))
+        result = xslt(etree.XML(_bytes('<a/>')))
         root = result.getroot()
         self.assertEquals(root.tag,  'test')
         self.assertEquals(len(root), 4)
@@ -982,7 +1173,7 @@ class ETreeXSLTTestCase(HelperTestCase):
 
     def test_exslt_regexp_match1(self):
         # taken from http://www.exslt.org/regexp/functions/match/index.html
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(etree.XML(_bytes("""\
 <xsl:stylesheet version="1.0"
    xmlns:regexp="http://exslt.org/regular-expressions"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -996,8 +1187,8 @@ class ETreeXSLTTestCase(HelperTestCase):
     </test>
   </xsl:template>
 </xsl:stylesheet>
-"""))
-        result = xslt(etree.XML('<a/>'))
+""")))
+        result = xslt(etree.XML(_bytes('<a/>')))
         root = result.getroot()
         self.assertEquals(root.tag,  'test')
         self.assertEquals(len(root), 5)
@@ -1018,7 +1209,7 @@ class ETreeXSLTTestCase(HelperTestCase):
 
     def test_exslt_regexp_match2(self):
         # taken from http://www.exslt.org/regexp/functions/match/index.html
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(self.parse("""\
 <xsl:stylesheet version="1.0"
    xmlns:regexp="http://exslt.org/regular-expressions"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -1032,7 +1223,7 @@ class ETreeXSLTTestCase(HelperTestCase):
   </xsl:template>
 </xsl:stylesheet>
 """))
-        result = xslt(etree.XML('<a/>'))
+        result = xslt(etree.XML(_bytes('<a/>')))
         root = result.getroot()
         self.assertEquals(root.tag,  'test')
         self.assertEquals(len(root), 5)
@@ -1046,7 +1237,7 @@ class ETreeXSLTTestCase(HelperTestCase):
     def _test_exslt_regexp_match3(self):
         # taken from http://www.exslt.org/regexp/functions/match/index.html
         # THIS IS NOT SUPPORTED!
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(etree.XML(_bytes("""\
 <xsl:stylesheet version="1.0"
    xmlns:regexp="http://exslt.org/regular-expressions"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -1059,8 +1250,8 @@ class ETreeXSLTTestCase(HelperTestCase):
     </test>
   </xsl:template>
 </xsl:stylesheet>
-"""))
-        result = xslt(etree.XML('<a/>'))
+""")))
+        result = xslt(etree.XML(_bytes('<a/>')))
         root = result.getroot()
         self.assertEquals(root.tag,  'test')
         self.assertEquals(len(root), 4)
@@ -1073,7 +1264,7 @@ class ETreeXSLTTestCase(HelperTestCase):
     def _test_exslt_regexp_match4(self):
         # taken from http://www.exslt.org/regexp/functions/match/index.html
         # THIS IS NOT SUPPORTED!
-        xslt = etree.XSLT(etree.XML("""\
+        xslt = etree.XSLT(etree.XML(_bytes("""\
 <xsl:stylesheet version="1.0"
    xmlns:regexp="http://exslt.org/regular-expressions"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
@@ -1086,8 +1277,8 @@ class ETreeXSLTTestCase(HelperTestCase):
     </test>
   </xsl:template>
 </xsl:stylesheet>
-"""))
-        result = xslt(etree.XML('<a/>'))
+""")))
+        result = xslt(etree.XML(_bytes('<a/>')))
         root = result.getroot()
         self.assertEquals(root.tag,  'test')
         self.assertEquals(len(root), 4)
@@ -1097,14 +1288,77 @@ class ETreeXSLTTestCase(HelperTestCase):
         self.assertEquals(root[2].text, "a")
         self.assertEquals(root[3].text, "test")
 
+
+class Py3XSLTTestCase(HelperTestCase):
+    """XSLT tests for etree under Python 3"""
+    def test_xslt_result_bytes(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*" />
+  <xsl:template match="/">
+    <foo><xsl:value-of select="/a/b/text()" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st(tree)
+        self.assertEquals(_bytes('''\
+<?xml version="1.0"?>
+<foo>B</foo>
+'''),
+                          bytes(res))
+
+    def test_xslt_result_bytearray(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*" />
+  <xsl:template match="/">
+    <foo><xsl:value-of select="/a/b/text()" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st(tree)
+        self.assertEquals(_bytes('''\
+<?xml version="1.0"?>
+<foo>B</foo>
+'''),
+                          bytearray(res))
+
+    def test_xslt_result_memoryview(self):
+        tree = self.parse('<a><b>B</b><c>C</c></a>')
+        style = self.parse('''\
+<xsl:stylesheet version="1.0"
+    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
+  <xsl:template match="*" />
+  <xsl:template match="/">
+    <foo><xsl:value-of select="/a/b/text()" /></foo>
+  </xsl:template>
+</xsl:stylesheet>''')
+
+        st = etree.XSLT(style)
+        res = st(tree)
+        self.assertEquals(_bytes('''\
+<?xml version="1.0"?>
+<foo>B</foo>
+'''),
+                          bytes(memoryview(res)))
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeXSLTTestCase)])
+    if is_python3:
+        suite.addTests([unittest.makeSuite(Py3XSLTTestCase)])
     suite.addTests(
-        [doctest.DocFileSuite('../../../doc/extensions.txt')])
+        [make_doctest('../../../doc/extensions.txt')])
     suite.addTests(
-        [doctest.DocFileSuite('../../../doc/xpathxslt.txt')])
+        [make_doctest('../../../doc/xpathxslt.txt')])
     return suite
 
 if __name__ == '__main__':
-    print 'to test use test.py %s' % __file__
+    print('to test use test.py %s' % __file__)
